VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cParallelPort"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False

' ***************************************************************************************
'     NAME: Parallel Port
'     DESC: Klasse zum Handling eines Parallel-Ports
' ***************************************************************************************
'
'     AUTHOR:  Stefan Maag
'     CREATE:  22/11/20012
'     CHANGE:
'     COPY:    Dieser Code ist frei verfgbar. Die Verwendung des Codes
'     COPY:    oder Codeteilen erfolgt immer auf eigenes Risiko und Verantwortung

' ===========================================================================
'  REM: Quellenangaben:
' ===========================================================================
'  REM:
'  REM:  1: www.activevb.de - Tutorials
'  REM:  2: MSDN Library Juli 2001, Microsoft
'  REM:  3: Jan's Parallel Port FAQ, www.logix4u.cjb.net
'  REM:  4: Interfacing the Standard Parallel Port, www.beyondlogic.org
'  REM:
' ===========================================================================
'  REM: Weitere bentigte Dateien:
' ===========================================================================
'
'  REM: Die inpout32.dll in der Version fr Windows 9x und NT/2k


' ===========================================================================
'  LPT-Port Adressen
' ===========================================================================
'
'  ein LPT-Port ist an 3 Adressen mglich, normalerweise wie folgt, es ist
'  jedoch mglich, dass LPT1 nicht an 378h zu finden ist sondern an einer
'  der beiden Anderen. Die Adresse 3BCh ist angeblich kritisch, da
'  diese teilweise auch fr Video statt LPT verwendet werden kann!

'     378h  : LPT1
'     278h  : LPT2
'     3BCh  : reserviert fr in Videoadapter integrierte LPT-Controller
'             hier werden keine zustzlichen ECP/EPP-Register untersttzt
'

'  Das Bios schreibt die PortAdressen der LPT's in den Speicher:
'  nur leider kann man das zumidest unter Windows-NT-Plattformen
'  nicht direkt mit CopyMemory auslesen
'
'     408h  : LPT1 Basisadresse
'     40Ah  : LPT2 Basisadresse
'     40Ch  : LPT3 Basisadresse
'     40Eh  : LPT4 Basisadresse - 40Eh wird in PS/2 oder neuerem Bios meist fr
'                                 die Extented Bios Area verwendet und ist daher
'                                 fr LPT ungltig
'

' ===========================================================================
'   Zuordnung LPT-Port-Pins zu den Register-Bits
' ===========================================================================

' Register 0 = DatenRegister   : Adr 378
' Register 1 = StatusRegister  : Adr 379
' Register 2 = ControlRegister : Adr 37A

' BASE ADDRESS  (PDR) Data register (Read or Write)

'  Bit 0 - Pin 2 - Data Output 1
'  Bit 1 - Pin 3 - Data Output 2
'  Bit 2 - Pin 4 - Data Output 3
'  Bit 3 - Pin 5 - Data Output 4
'  Bit 4 - Pin 6 - Data Output 5
'  Bit 5 - Pin 7 - Data Output 6
'  Bit 6 - Pin 8 - Data Output 7
'  Bit 7 - Pin 9 - Data Output 8

' BASE ADDRESS + 1     (PSR) Status register (Read Only)

'  Bit 0 -        - Not used
'  Bit 1 -        - Not used
'  Bit 2 -        - Not used
'  Bit 3 - Pin 15 - ERROR Input
'  Bit 4 - Pin 13 - SELECT Input
'  Bit 5 - Pin 12 - PAPER END Input
'  Bit 6 - Pin 10 - ACKNOWLEDGE Input
'  Bit 7 - Pin 11 - BUSY Input

' BASE ADDRESS + 2     (PCR) Control register (Read or Write)

'  Bit 0 - Pin  1 - DATA STROBE Output
'  Bit 1 - Pin 14 - AUTO LINE FEED Output
'  Bit 2 - Pin 16 - INITIALIZE PRINTER Output
'  Bit 3 - Pin 17 - SELECT INPUT Output
'  Bit 4 -        - 0 = disable interrupt, 1 = enable interrupt
'  Bit 5 -
'  Bit 6 -
'  Bit 7 -


' ===========================================================================
'  SPP - Register
' ===========================================================================

' ------------------------------------------------------------------------
' Address         |  Register Name                       |  Read/Write
' ------------------------------------------------------------------------
' Base            |  Data Register                       |  Read/Write
' Base + 1        |  Status Register                     |  (Read)/Write
' Base + 2        |  Control Register                    |  Read/Write
' ------------------------------------------------------------------------

' ========================================================================
'           Status Register Bit Definitions : Base + 1
' ------------------------------------------------------------------------
'  Bit   |  Function                |  Low                  | High
' ------------------------------------------------------------------------
'  7     |  Busy (Low-Active!!!)    |  Busy                 | Not Busy
'  6     |  Acknowledge             |  Nack                 | Ack
'  5     |  Paper Status            |  No Paper             | Paper
'  4     |  Selection Status        |  Not Selected         | Selected
'  3     |  Error Status            |  No Error             | Error
'  2     |  Not Used                |  -                    | -
'  1     |  Not Used                |  -                    | -
' ------------------------------------------------------------------------
'  0     |  SPP: Not Used           |  -                    | -
'        |  EPP: TimeOut            |                       |
' ------------------------------------------------------------------------
' ACHTUNG: Bit 7 des Statusregisters (Busy) ist Low-Active, d.h. +5 Volt
'          auf dieser Leitung fhren zu einer 0 im Bit, 0V zu einer 1.
'          Alle anderen Bits sind High-Active
' ========================================================================


' ========================================================================
'           Control Register Bit Definitions : Base + 2
' ------------------------------------------------------------------------
'  Bit   |  Function                |  Low                  | High
' ------------------------------------------------------------------------
'  7     |  (MSB) Not Used          |  -                    | -
'  6     |  Not Used                |  -                    | -
' ------------------------------------------------------------------------
'  5     |  SPP     : Not Used      |  -                    | -
'        |  EPP/ECP : DataMode      | Output                | Input
' ------------------------------------------------------------------------
'  4     |  Interrupt Control       | Interrupts Disabled   | Interrupts Enabled
'  3     |  Select                  | Not Selected          | Selcted
'  2     |  Initialize              | False                 | True
'  1     |  Auto Feed               | True                  | False
'  0     | (LSB) Strobe (Active-Low)| True                  | False
' ------------------------------------------------------------------------
' ACHTUNG: Bit 5 des Controlregisters ist im Standard SPP Mode nicht
'          verwendet, im falle eines Bidirektionalen Ports schaltet
'          es zwischen Daten Output und Input um!
' ========================================================================


' ========================================================================
'                             ECP - Register
' ========================================================================

' ------------------------------------------------------------------------
' Address         |  Port Name                           |  Read/Write
' ------------------------------------------------------------------------
' Base + 0        |  Data Port (SPP)                     |  Write
'                 |  ECP Address FIFO (ECP MODE)         |  Read/Write
' ------------------------------------------------------------------------
' Base + 1        |  Status Port (All Modes)             |  Read/Write
' ------------------------------------------------------------------------
' Base + 2        |  Control Port (All Modes)            |  Read/Write
' ------------------------------------------------------------------------
' Base + 400h     |  Data FIFO (Parallel Port FIFO Mode) |  Read/Write
'                 |  Data FIFO (ECP Mode)                |  Read/Write
'                 |  Test FIFO (Test Mode)               |  Read/Write
'                 |  Configuration Register A            |
'                 |  (Configuration Mode)                |  Read/Write
' ------------------------------------------------------------------------
' Base + 401h     |  Configuration Register B            |
'                 |  (Configuration Mode)                |  Read/Write
' ------------------------------------------------------------------------
' Base + 402h     |  Extended Control Register (ECR)     |
'                 |  (Used by all modes)                 |  Read/Write
' ========================================================================


' ========================================================================
'       Funktionen des Extended Control Register (ECR) : Base + 402h
' ------------------------------------------------------------------------
'  Bit      |  Function
' ------------------------------------------------------------------------
'  7:5      |  Selects Current Mode of Operation
'           |  000    Standard Mode, SPP
'           |  001    Byte Mode
'           |  010    Parallel Port FIFO Mode, Fast Centronics
'           |  011    ECP FIFO Mode
'           |  100    EPP Mode
'           |  101    Reserved
'           |  110    FIFO Test Mode
'           |  111    Configuration Mode
' ------------------------------------------------------------------------
'  4        |  ECP Interrupt Bit
' ------------------------------------------------------------------------
'  3        |  DMA Enable Bit
' ------------------------------------------------------------------------
'  2        |  ECP Service Bit
' ------------------------------------------------------------------------
'  1        |  FIFO Full
' ------------------------------------------------------------------------
'  0 (LSB)  |  FIFO Empty
' ========================================================================


' ========================================================================
'     ECP Configuration Register A : Base + 400h : nur im Config-Mode
' ------------------------------------------------------------------------
'  Bit      |  Function
' ------------------------------------------------------------------------
'  7        |  1 : Interrupts are level triggered
'           |  0 : Interrupts are edge triggered (Pulses)
' ------------------------------------------------------------------------
'  6:4      |  00h      Accepts Max. 16 Bit wide words
'           |  01h      Accepts Max.  8 Bit wide words
'           |  02h      Accepts Max. 32 Bit wide words
'           |  03h:07h  Reserved for future expansion
' ------------------------------------------------------------------------
'  3        |  Reserved
' ------------------------------------------------------------------------
'  2        |  Host Recovery : Pipeline/Transmitter Byte included in FIFO?
'           |  0 : In forward direction, the 1 byte in the
'           |      transmitter pipeline doesn't affect FIFO Full.
'           |  1 : In forward direction, the 1 byte in the
'           |      transmitter pipeline is include as part of FIFO Full.
' ------------------------------------------------------------------------
'  1:0 (LSB)|  Host Recovery : Unsent byte(s) left in FIFO
'           |  00    Complete Pword
'           |  01    1 Valid Byte
'           |  10    2 Valid Bytes
'           |  11    3 Valid Bytes
' ========================================================================


' ========================================================================
'  ECP Configuration Register B : Base + 401h : nur im Config-Mode
' ------------------------------------------------------------------------
'  Bit      |  Function
' ------------------------------------------------------------------------
'  7        |  1  Compress outgoing Data Using RLE
'           |  0  Do Not compress Data
' ------------------------------------------------------------------------
'  6        |  Interrupt Status - Shows the Current Status of the IRQ Pin
' ------------------------------------------------------------------------
'  5:3      | Selects or Displays Status of Interrupt Request Line.
'           |  000   Interrupt Selected Via Jumper
'           |  001   IRQ 7
'           |  010   IRQ 9
'           |  011   IRQ 10
'           |  100   IRQ 11
'           |  101   IRQ 14
'           |  110   IRQ 15
'           |  111   IRQ 5
' ------------------------------------------------------------------------
'  2:0 (LSB)| Selects or Displays Status of the DMA Channel the Printer Card Uses
'           |  000   Uses a Jumpered 8 Bit DMA Channel
'           |  001   DMA Channel 1
'           |  010   DMA Channel 2
'           |  011   DMA Channel 3
'           |  100   Uses a Jumpered 16 Bit DMA Channel
'           |  101   DMA Channel 5
'           |  110   DMA Channel 6
'           |  111   DMA Channel 7
' ========================================================================


' ===========================================================================
'  EPP - Register
' ===========================================================================

' ------------------------------------------------------------------------
' Address         |  Port Name                           |  Read/Write
' ------------------------------------------------------------------------
'  Base + 0       |  Data Port    (SPP)                  |  Write
'  Base + 1       |  Status Port  (SPP)                  |  Read
'  Base + 2       |  Control Port (SPP)                  |  Write
'  Base + 3       |  Address Port (EPP)                  |  Read/Write
'  Base + 4       |  Data Port    (EPP)                  |  Read/Write
'  Base + 5       |  Undefined (16/32bit Transfers)      |
'  Base + 6       |  Undefined (32bit Transfers)         |
'  Base + 7       |  Undefined (32bit Transfers)         |
' ------------------------------------------------------------------------

' ===========================================================================
' Welcher Porttyp?
'                    SPP : Standard Parallel Port
'                    EPP : Extended Parallel Port
'                    ECP : Extended Centronics Port
' ===========================================================================

' - bersetztung aus Jan's Parallelport FAQ

' SPP (original), PS/2 (simple bidirectional), EPP, or ECP.
' Es gibt also verschiedene Porttypen und somit auch unterschiedliche Ansteuerungen
' Der Test erkennt nur den PortTyp der aktiviert ist. Wenn die erweiterten Einstellungen
' des ControllerChips abgeschaltet sind, werden diese nicht erkannt.
' Eigentlich alle neuen Controller bieten die Mglichkeit den Standart SPP-Mode zu simulieren
' Nun denkt man, man beginnt mit der Prfung auf SPP und arbeitet sich langsam bis zum ECP hoch.
' Aber wenn der ECP Controller im Simulationmodus fr SPP ist, fllt der Test auf  PS/2 bidirectional
' negativ aus.
'
' Also geht man den umgekehrten Weg und Testet zuerst auf ECP und arbeitet sich nach unten.
' So wie es Microsoft ECP Dokument (auf der Developer's Network CD) beschreib.
'
' *** ECP erkennen: ***
'
' 1. extended ECP Control-Register (ECR) an der BasisAdresse + 402h lesen und berprfen, ob
'    Bit 0 (fifo emty) =1 und Bit 1 (fifo full) =0 ist. Diese Bits sollten getrennt von den
'    Bits 0 und 1 im Control-Register an der BasisAdresse + 2 sein. Man kann das berprfen,
'    indem man eines der Bits im Control-Register setzt und schaut, ob sich das entsprechende
'    Bit im ECP Register ndert.
'
' 2. Ein weiterer Test ist 34h ins ECR-Register zu schreiben und zurckzulesen. Die Bits 0 und 1
'    sind ReadOnly. Also wenn man 35h zurckliest hat man einen ECP-Port. Wenn man nun einen ECP
'    Port hat, kann man den internen Mode im ECP-Register setzen

'    ACHTUNG: ich habe festgestellt, da man das Schreiben eines wilkrlichen Wertes in das
'             ECP-Control-Register bleiben lassen sollte, wenn man nachher nicht explicit
'             einen der folgenden Modes setzt. Da 34h den ECP mode setzt, unabhngig ob vorher
'             etwas anderes eingestellt war!

'     Interne ECP Modes:
'
'     000 SPP
'     001 Byte
'     010 Fast Centronics
'     011 ECP
'     100 EPP
'     101 Reserved
'     110 Test
'     111 Config


' *** EPP erkennen: ***
'
' zustzlich zu den 3 Registern des Standard SPP hat ein ECP 4 weitere Register
' an der BasisAdresse +3 bis +6. Diese zustzlichen Register bieten eine Mglichkeit
' auf EPP zu testen, indem man einige Werte in eines der EPP-Register schreibt und
' zurckliest, hnlich wie man die Standardregister des SPP testet.
' Sind die Lesezugriffe erfolgreich, hat man wahrscheinlichst einen EPP Port.
' Es ist nicht ganz sicher, ob dieser Test bei allen EPPs arbeitet, da das EPP
' Handshake nicht komplett ist. Es gibt also keine Garantie fr den Inhalt der Register
' nach dem Transfer-Timeout. Sicherheitshalber lscht man das EPP Timeout-Bit
' (Bit 0 des Statusports an BasisAdresse +1) nach jedem Schreib- oder Lesezugriff.
' Leider variert die Methode zum lschen des Bits mit den Controllern. Bei einigen
' Ports lscht man indem man eine 1 schreibt, andere lschen ganz einfach automatisch
' beim lesen des Statusregisters. Aber es gibt eigentlich keine, die das Bit in
' konventioneller Weise lschen, indem man eine 0 schreibt. Trotzdem sollte man dies
' auch tun, um ganz sicher zu gehen.

' Achtung 1: bei SMC chips (vielleicht auch anderen) kann ein gesetztes Timeoutbit den
'            Port bis zu einem Hardwarereset oder unbrachbar machen. Oder man lscht das Bit wieder

' Achtung 2: Man sollte niht versuchen an der Adresse 3BCh auf EPP zu Testen. Dieses Register ist
'            nicht immer an dieser Adresse verfgbar und kann fr Video verwendet werden.


' *** SPP erkennen: ***
'
' Fr den SPP Test nimmt man auch die Try And Error Methode, indem man zwei Werte in das
' Datenregister schreibt und zurckliest. Dieser Test prft nur das Datenregister nicht
' die Status und Control-Register. Dies sollte aber trotzdem reichen.
' die anderen Port-Typen EPP, ECP und PS/2 sollten diesen Test auch bestehen.

' *** PS/2 Port erkennen: ***
'
' Um einen einfachen Test auf Bidirectional auszufhren versucht man zuerst den Port
' in den Inputmode zu schalten, indem man eine 1 in Bit 5 des Control-Register
' (BasisAdresse + 2) schreibt. Ist der Port bidirektional, so geht der Portoutput in
' TriState. Dann schreibt man 2 Werte in das Datenregister und liest jeden zurck.
' Wenn der Ausgang TriState ist, dann schlagen die Leseversuche fehl und man hat
' meist sicher einen bidirektionalen Port. Liest man hingegen zurck, was man geschieben
' hat, verblieb der Port im Output-Zustand und ist nicht bidirektional.
' Aber man sollte nur auf PS/2 testen, nachdem man festgestellt hat, da ein Port
' SPP, EPP oder ECP vorhanden ist, da man einen Fehler dazu verwendet um PS/2 zu erkennen.
' Ein nicht vorhandener Port wrde den Test auf PS/2 bestehen!

Option Explicit

Private Declare Function Inp Lib "inpout32.dll" Alias "Inp32" (ByVal PortAddress As Integer) As Integer
Private Declare Sub Out Lib "inpout32.dll" Alias "Out32" (ByVal PortAddress As Integer, ByVal Value As Integer)

Public Enum eLptType
   ' DESC: Aufzhlung der verschiedenen LptPort-Modes
   LPT_notPresent = 0            ' DESC: LPT Port ist nicht vorhanden
   LPT_SPP = 1                   ' DESC: SPP Standard Parallel Port
   LPT_EPP = 2                   ' DESC: EPP-Port : Extended Parallel Port
   LPT_ECP = 3                   ' DESC: ECP-Port : Extended Centronics Port
End Enum

Public Enum ePortRegister
   ' DESC: Aufzhlung der PortRegister, der Wert entspircht dem Offset zur BasisAdresse
   SPP_DataRegister = 0                ' DESC: Offset des Standard Daten-Register (All Modes)
   SPP_StatusRegister = 1              ' DESC: Offset des Standard Staus-Register (All Modes)
   SPP_Controlregister = 2             ' DESC: Offset des Standard Control-Register (All Modes)
   EPP_AddressRegister = 3             ' DESC: Offset des EPP Adress Register (nur EPP)
   EPP_DataRegister1 = 4               ' DESC: Offset des EPP Daten-Register 1 (nur EPP)
   EPP_DataRegister2 = 5               ' DESC: Offset des EPP Daten-Register 2 (nur EPP)
   EPP_DataRegister3 = 6               ' DESC: Offset des EPP Daten-Register 3 (nur EPP)
   EPP_DataRegister4 = 7               ' DESC: Offset des EPP Daten-Register 4 (nur EPP)
   ECP_DataFifoRegister = &H400        ' DESC: Offset des ECP FIFO-Daten-Register (nur ECP)
   ECP_ExtendedControlRegister = &H402 ' DESC: Offset des Extended-Control-Register
End Enum

Public Enum eECPinternalMode
   ' DESC: Aufzhlung fr die internen Modi von ECP-Ports. ECP-Ports untersttzen eine Reihe weiterer Modes
   ECP_Mode_SPP = 0              ' DESC: Standard SPP (normal)
   ECP_Mode_Byte = 1             ' DESC: Byte-Mode
   ECP_Mode_FastCentronic = 2    ' DESC: Fast Centronic - PS/2 Bidirectional
   ECP_Mode_ECP = 3              ' DESC: Extended Centronic Port
   ECP_Mode_EPP = 4              ' DESC: EPP
   ECP_Mode_Reserved = 5         ' DESC: reserviert
   ECP_Mode_FifoTest = 6         ' DESC: FIFO Test Modus
   ECP_Mode_Config = 7           ' DESC: Configurations Modus
End Enum

' intern verwendete Portadresse als Integer, da DLL einen Int verwendet
Private m_PortAddress As Integer
Private b_Init As Boolean                 ' interne Meldung Initialisierung wurde durchlaufen
Private m_Error As Long                   ' interner Speicher fr aufgetretenen Fehler

' interne Meldungen fr Register vorhanden
Private m_ECP_RegisterPresent As Boolean
Private m_EPP_RegisterPresent As Boolean
Private m_SPP_RegisterPresent As Boolean

Private m_IsBidirectional As Boolean      ' interne Meldung Port ist bidirektional

' True: I/O Mode ist Output, False: I/O Mode ist Input - nur bei bidirektional mglich
Private m_DataModeOutput As Boolean
'
Public Property Get PortAddress() As Integer
' ===========================================================================
'  NAME:    PortAddress
'  DESC:    Portadresse des zu verwendenden LPT-Ports
'  RETURN:  aktuell eingestellte Adresse
' ===========================================================================
   PortAddress = m_PortAddress
End Property

Public Property Let PortAddress(ByVal New_Address As Integer)
' ===========================================================================
'  NAME:    PortAddress
'  DESC:    Portadresse des zu verwendenden LPT-Ports
'  PARA(New_Address):  neue Port Adresse
' ===========================================================================
   If New_Address <> m_PortAddress Then
      m_PortAddress = New_Address
      CheckPort ' Port neu berprfen, da sich Adresse gendert hat
   End If
End Property

Public Property Get ECP_RegisterPresent() As Boolean
' ===========================================================================
'  NAME:    ECP_RegisterPresent
'  DESC:    gibt an, ob ECP-Register vorhanden sind
'  RETURN:  True, wenn Register vorhanden - Es handelt sich um einen ECP-Port
' ===========================================================================
   
   If Not b_Init Then CheckPort                 ' falls ntig, Port prfen
   ECP_RegisterPresent = m_ECP_RegisterPresent
End Property

Public Property Get EPP_RegisterPresent() As Boolean
' ===========================================================================
'  NAME:    EPP_RegisterPresent
'  DESC:    gibt an, ob EPP-Register vorhanden sind
'  RETURN:  True, wenn Register vorhanden - Es handelt sich um einen EPP-Port
' ===========================================================================

   If Not b_Init Then CheckPort                 ' falls ntig, Port prfen
   EPP_RegisterPresent = m_EPP_RegisterPresent
End Property

Public Property Get SPP_RegisterPresent() As Boolean
' ===========================================================================
'  NAME:    SPP_RegisterPresent
'  DESC:    gibt an, ob SPP-Register vorhanden sind. Also berhaupt ein Port
'  RETURN:  True, wenn Register vorhanden
' ===========================================================================

   If Not b_Init Then CheckPort                 ' falls ntig, Port prfen
   SPP_RegisterPresent = m_SPP_RegisterPresent
End Property


Public Property Get PortExist() As Boolean
' ===========================================================================
'  NAME:    PortExist
'  DESC:    gibt an, ob an der Adresse ein Port Existiert.
'  DESC:    Es wird auf die Prsenz aller 3 Registertypen verglichen, da
'  DESC:    ein ECP-Port im ECP-FIFO Modus, den SPP Test nicht unbedingt besteht
'  DESC:    da dabei das SPP-Datenregister zum ECP Address-FIFO wird
'  RETURN:  True, wenn Port bzw. Standard SPP-Register vorhanden sind
' ===========================================================================
   If Not b_Init Then CheckPort                 ' falls ntig, Port prfen
   PortExist = m_SPP_RegisterPresent Or m_EPP_RegisterPresent Or m_ECP_RegisterPresent
End Property

Public Property Get IsBidirectional() As Boolean
' ===========================================================================
'  NAME:    IsBidirectional
'  DESC:    gibt an, ob Port Bidirectional ist
'  RETURN:  True, wenn bidirectional
' ===========================================================================
   If Not b_Init Then CheckPort                 ' falls ntig, Port prfen
   IsBidirectional = m_IsBidirectional
End Property


Public Property Get ECP_internalMode() As eECPinternalMode
' ===========================================================================
'  NAME:    ECPMode
'  DESC:    interner Mode eines ECP-Ports
'  RETURN:  interner Mode des ECP-Ports: Werte 0 bis 7
' ===========================================================================
   If Not b_Init Then CheckPort                    ' falls ntig, Port prfen
   
   If m_ECP_RegisterPresent Then                   ' Falls ECP-Port
      ECP_internalMode = GetECPMode(m_PortAddress) ' Wert ECP-Mode lesen
   Else
      m_Error = 2                                  ' kein ECP vorhanden
   End If
End Property

Public Property Let ECP_internalMode(ByVal New_Mode As eECPinternalMode)
' ===========================================================================
'  NAME:    ECPMode
'  DESC:    interner Mode eines ECP-Ports
'  PARA(New_Mode): Mode des ECP-Ports: Werte 0 bis 7 als eECPinternalMode
' ===========================================================================
   If Not b_Init Then CheckPort                 ' falls ntig, Port prfen
   
   If m_ECP_RegisterPresent Then                ' Falls ECP-Port
      SetECPMode m_PortAddress, New_Mode        ' ECP-Mode setzen
   Else
      m_Error = 2                               ' kein ECP vorhanden
   End If

End Property

Public Property Get DataModeOutput() As Boolean
' ===========================================================================
'  NAME:    DataModeOutput
'  DESC:    Port ist im Output Mode
'  RETURN:  True = Output Mode, False = Input Mode
' ===========================================================================
   
   DataModeOutput = m_DataModeOutput
End Property

Public Property Let DataModeOutput(ByVal New_Value As Boolean)
' ===========================================================================
'  NAME:    DataModeOutput
'  DESC:    Portmodus Input/Output festlegen
'  PARA(New_Value): True = Output Mode, False = Input Mode
' ===========================================================================
   Dim b As Integer
   ' Mode ist schon eingestellt
   If m_DataModeOutput = New_Value Then Exit Property
   
   m_DataModeOutput = New_Value
   
   b = Inp(m_PortAddress + 2)          ' Controlregister lesen
   
   If New_Value Then
      b = b And Not 35                ' Bit 0,1,5 rcksetzen, bei Output ist Bit5=0
   Else
      b = b And Not 3
      b = b Or 32                      ' Bit 5 setzen, bei Input ist Bit = 1
   End If
   Out m_PortAddress, 0                ' Datenregister lschen
   Out m_PortAddress + 2, b            ' neuen Wert in Control-Register schreiben
End Property


Public Property Get RegisterValue(ByVal Register As ePortRegister) As Integer
' ===========================================================================
'  NAME:    RegisterValue
'  DESC:    lesen/schreiben des gewhlten Registers
'  PARA(Register): Adress Offset des gewnschten Registers
'  RETURN:  Inhalt des Registers
' ===========================================================================
   
   ' Register lesen: PortAddress + AddressOffset des Registers
   RegisterValue = Inp(m_PortAddress + Register)
End Property

Public Property Let RegisterValue(ByVal Register As ePortRegister, ByVal New_Value As Integer)
   ' Register schreiben: PortAddress + AddressOffset des Registers
   Out m_PortAddress + Register, New_Value
End Property

Public Property Get Error() As Long
' ===========================================================================
'  NAME:    Error
'  DESC:    gibt die letzte Fehlernummer aus, bzw. dient zum lschen des Fehlers
'  DESC:    0 = kein Fehler
'  DESC:    1 = kein Port vorhanden
'  DESC:    2 = kein ECP vorhanden
'  DESC:    3 = kein EPP vorhanden
'  RETURN:  Fehlerwert
' ===========================================================================
   Error = m_Error
End Property

Public Property Let Error(ByVal New_Error As Long)
   m_Error = New_Error
End Property

Public Function GetLptPortType() As eLptType
' ===========================================================================
'  NAME:    GetLptPortType
'  DESC:    gibt den Typ des LPT-Ports an
'  RETURN:  LPT Port Typ als eLptType
' ===========================================================================
   
   If Not b_Init Then CheckPort                 ' wenn noch nicht intitialisiert
   
   If m_ECP_RegisterPresent Then                ' ECP-Port
      GetLptPortType = LPT_ECP
   ElseIf m_EPP_RegisterPresent Then            ' EPP-Port
      GetLptPortType = LPT_EPP
   ElseIf m_SPP_RegisterPresent Then            ' SPP-Port
      GetLptPortType = LPT_SPP
   Else                                         ' Port not Present
      GetLptPortType = LPT_notPresent
   End If
   
 End Function

Private Function CheckRegisterECP(ByVal PortADR As Integer) As Boolean
' ===========================================================================
'  CheckRegisterECP: prft ob der Port ber ECP-Register verfgt,
'                    also ein ECP-Port ist
' ===========================================================================
   Const bTest1 = &HAA              ' = 1010 1010
   Const bTest2 = &H55              ' = 0101 0101
   Dim b As Integer
   Dim b1 As Byte, b2 As Byte       ' Variablen fr gelesene Werte von Port
   
'   ' ControlRegister schreiben, Port sicherheitshalber auf Ausgangs-Mode setzen
'   b = Inp(PortADR + 2)             ' Controlregister lesen
'   b = b And Not 35                 ' Bit 0,1,5 des Registers rcksetzen
'   Out PortADR + 2, b               ' Wert ins Controlregister zurck
  
   b1 = Inp(PortADR + &H402)        ' ECP ControlRegister (ECR) lesen
   
   ' Bit 0 FIFO leer = 1 und Bit 2 FIFO voll = 0 prfen
   If (b1 And 1) = 1 And (b1 And 2) = 0 Then
      CheckRegisterECP = True
   Else
      
      Out PortADR + &H402, 0        ' versucht den ECP Mode 0 = Standard SPP zu setzen
      
      b = Inp(PortADR + &H402)      ' ECR-Register erneut lesen

      If (b And 1) = 1 And (b And 2) = 0 Then CheckRegisterECP = True
   
   End If

End Function

Private Function CheckRegisterEPP(ByVal PortADR As Integer) As Boolean
' ===========================================================================
'  CheckRegisterEPP: prft ob der Port ber EPP-Register verfgt,
'                    also ein EPP-Port ist
' ===========================================================================

   Const bTest1 = &HAA              ' = 1010 1010
   Const bTest2 = &H55              ' = 0101 0101
   Dim b1 As Byte, b2 As Byte       ' Variablen fr gelesene Werte von Port
   
   Out PortADR + 3, bTest1          ' erstes ECP-Register beschreiben (Adress-Register)
   b1 = Inp(PortADR + 3)            ' Wert wieder zurcklesen
   Out PortADR + 3, bTest2
   b2 = Inp(PortADR + 3)
   
   If (b1 = bTest1 And b2 = bTest2) Then
      
      CheckRegisterEPP = True
      
      ' Timeout-Bit sicherheitshalber lschen, die Methode variert bei verschieden Controllern
      Out PortADR + 1, 1       ' Timeout lschen Methode schreiben von 1
      Out PortADR + 1, 0       ' Timeout lschen Methode schreiben von 0
      Call Inp(PortADR + 1)    ' Timeout lschen Methode Statusregister lesen

   End If

End Function

Private Function CheckRegisterSPP(ByVal PortADR As Integer) As Boolean
' ===========================================================================
'  CheckRegisterSPP: prft ob berhaupt an der Adresse ein Parallelport
'                    vorhanden ist. Dies ist gleichzeitig der
'                    Standard SPP-Test, den auch EPP und ECP
'                    Ports bestehen mssen
' ===========================================================================

   Const bTest1 = &HAA              ' = 1010 1010
   Const bTest2 = &H55              ' = 0101 0101
   Dim b As Integer
   Dim b1 As Byte, b2 As Byte       ' Variablen fr gelesene Werte von Port
   
   ' prfen ob berhaupt an der Adresse ein Parallelport vorhanden ist
   ' dies ist gleichzeitig der Standard SPP-Test, den auch EPP und ECP Ports bestehen mssen
   
   ' ------ SPP Register Test ------------------------------------------------------
   
   ' Port auf Ausgangs-Mode setzen, sonst ist Datenregister im Tristate
   b = Inp(PortADR + 2)    ' Controlregister lesen
   b = b And Not 32             ' Bit 0,1,5 des Registers rcksetzen
   Out PortADR + 2, b      ' ControlRegister schreiben,
   
   Out PortADR, 0          ' wieder 0 in Datenregister schreiben
   Out PortADR, bTest1     ' Testwert 1 in Datenregister schreiben
   b1 = Inp(PortADR)       ' Datenregister wieder auslesen
   Out PortADR, bTest2     ' Testwert 2 in Datenregister
   b2 = Inp(PortADR)       ' Datenregister wieder einlesen
   Out PortADR, 0          ' wieder 0 in Datenregister schreiben
   
   ' wenn Daten unvervlscht gelesen wurden existiert Port
   CheckRegisterSPP = (b1 = bTest1) And (b2 = bTest2)
   
End Function

Private Function CheckBidirectional(ByVal PortADR As Integer) As Boolean
' ===========================================================================
'  CheckBidirectional: prft ob der Port Bidirectional ist
'
'  ACHTUNG: nur auf Bidirectional prfen, wenn man sicher ist, da ein
'           Port vorhanden ist, da ein Lesefehler im Tristatemode zur Erkennung
'           verwendet wird. Ein nicht vorhandener Port besteht den
'           Bidirectional-Test
' ===========================================================================

   Const bTest1 = &HAA              ' = 1010 1010
   Const bTest2 = &H55              ' = 0101 0101
   Dim b As Integer
   Dim b1 As Byte, b2 As Byte       ' Variablen fr gelesene Werte von Port
   
   ' PS/2 Bidirectional Test
   
   b = Inp(PortADR + 2)       ' Controlregister lesen
   b = b Or 32                ' Bit 5 des Controlregisters setzen = InputMode
   Out PortADR + 2, b         ' Controlregister schreiben
   
   Out PortADR, bTest1        ' 1. Testwert schreiben
   b1 = Inp(PortADR)          ' Testwert zurcklesen
   Out PortADR, bTest2        ' 2. Testwert schreiben
   b2 = Inp(PortADR)          ' Testwert zurcklesen
   
   b = b And Not 32               ' Bit 5 des Controlregisters wieder rcksetzen =OutputMode
   Out PortADR + 2, b         ' Wieder auf AusgansMode setzen
   Out PortADR, 0             ' wieder 0 in Datenregister schreiben
   
   ' Wenn Werte korrekt gelesen wurden, dann ist Port nicht Bidirctional
   If Not (b1 = &HAA And b2 = 0) Then
      CheckBidirectional = True
   End If

End Function

Private Function GetECPMode(ByVal PortADR As Integer) As eECPinternalMode
' ===========================================================================
'  GetECPMode : gibt die interne Einstellung eines ECP-Ports zurck
'               Voraussetzung ist ein vorhandener ECP-Port
'
'     Interne ECP Modes:
'
'     000 SPP
'     001 Byte
'     010 Fast Centronics, PS/2 bidirektional
'     011 ECP
'     100 EPP
'     101 Reserved
'     110 Test FIFO
'     111 Config
' ===========================================================================
   Dim b As Byte
   
   b = Inp(PortADR + &H402)   ' ECP Control-Register lesen
   b = b \ 32                 ' ShiftRight 5
   GetECPMode = b
End Function

Private Function SetECPMode(ByVal PortADR As Integer, ByVal Mode As eECPinternalMode)
' ===========================================================================
'  SetECPMode : setzt die interne Einstellung eines ECP-Ports
'               Voraussetzung ist ein vorhandener ECP-Port
' ===========================================================================

   Dim b As Byte
   b = Mode And 7          ' BitMaske 0000 0111 : stellt gltige Werte sicher
   b = b * 32              ' ShiftLeft 5 : ECP Mode steht in den Bits 7-5
   Out PortADR + &H402, b  ' ECP Control Register schreiben
   CheckPort
End Function

Private Sub CheckPort()
' ===========================================================================
'  Check des Ports and PortAddress und setzen der Klasseninternen Meldungen
' ===========================================================================
   
   Dim b As Byte
   
   m_ECP_RegisterPresent = CheckRegisterECP(m_PortAddress)
   m_EPP_RegisterPresent = CheckRegisterEPP(m_PortAddress)
   m_SPP_RegisterPresent = CheckRegisterSPP(m_PortAddress)
   
   ' Wenn Port vorhanden, dann auf Bidirectional prfen
   If m_SPP_RegisterPresent Then
      m_IsBidirectional = CheckBidirectional(m_PortAddress)
   Else
      m_IsBidirectional = False
   End If
   
   If m_IsBidirectional Then
      b = Inp(m_PortAddress + 2)          ' SPP-ControlRegister lesen
      m_DataModeOutput = ((b And 32) = 0) ' wenn Bit 5 des Registers nicht gesetzt = Output
                                          ' sonst Input
   Else
      m_DataModeOutput = False
   End If
   
   b_Init = True           ' initialisierung wurde durchlaufen
   
End Sub

Private Sub Class_Initialize()
   m_PortAddress = &H378         ' Standard Portadresse fr LPT1
End Sub

